@@ -16,9 +16,7 @@
    General Public License for more details.
    
    You should have received a copy of the GNU General Public
-   License along with this program; if not, write to the
-   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   License along with this program; if not, see <http://www.gnu.org/licenses/>.
    
    Authors: Cosimo Cecchi <cosimoc@gnome.org>
 */
@@ -76,7 +74,7 @@ file_icons_changed (NautilusFile *file,
 	GdkPixbuf *pixbuf;
 
 	pixbuf = nautilus_file_get_icon_pixbuf (fcd->details->destination,
-						NAUTILUS_ICON_SIZE_LARGE,
+						NAUTILUS_CANVAS_ICON_SIZE_SMALL,
 						TRUE,
 						gtk_widget_get_scale_factor (fcd->details->dest_image),
 						NAUTILUS_FILE_ICON_FLAGS_USE_THUMBNAILS);
@@ -85,7 +83,7 @@ file_icons_changed (NautilusFile *file,
 	g_object_unref (pixbuf);
 
 	pixbuf = nautilus_file_get_icon_pixbuf (fcd->details->source,
-						NAUTILUS_ICON_SIZE_LARGE,
+						NAUTILUS_CANVAS_ICON_SIZE_SMALL,
 						TRUE,
 						gtk_widget_get_scale_factor (fcd->details->src_image),
 						NAUTILUS_FILE_ICON_FLAGS_USE_THUMBNAILS);
@@ -221,7 +219,7 @@ file_list_ready_cb (GList *files,
 
 	/* Set up file icons */
 	pixbuf = nautilus_file_get_icon_pixbuf (dest,
-						NAUTILUS_ICON_SIZE_LARGE,
+						NAUTILUS_CANVAS_ICON_SIZE_SMALL,
 						TRUE,
 						gtk_widget_get_scale_factor (fcd->details->titles_vbox),
 						NAUTILUS_FILE_ICON_FLAGS_USE_THUMBNAILS);
@@ -232,7 +230,7 @@ file_list_ready_cb (GList *files,
 	g_object_unref (pixbuf);
 
 	pixbuf = nautilus_file_get_icon_pixbuf (src,
-						NAUTILUS_ICON_SIZE_LARGE,
+						NAUTILUS_CANVAS_ICON_SIZE_SMALL,
 						TRUE,
 						gtk_widget_get_scale_factor (fcd->details->titles_vbox),
 						NAUTILUS_FILE_ICON_FLAGS_USE_THUMBNAILS);
@@ -626,6 +624,8 @@ nautilus_file_conflict_dialog_new (GtkWindow *parent,
 	GtkWidget *dialog;
 	
 	dialog = GTK_WIDGET (g_object_new (NAUTILUS_TYPE_FILE_CONFLICT_DIALOG,
+					   "use-header-bar", TRUE,
+					   "modal", TRUE,
 					   "title", _("File conflict"),
 					   NULL));
 	set_source_and_destination (dialog,
