@@ -24,6 +24,7 @@
 */
 
 #include <config.h>
+#include <stdlib.h>
 
 #include "nautilus-desktop-canvas-view.h"
 
@@ -257,6 +258,42 @@ nautilus_desktop_canvas_view_dispose (GObject *object)
 	G_OBJECT_CLASS (nautilus_desktop_canvas_view_parent_class)->dispose (object);
 }
 
+static void
+nautilus_desktop_canvas_view_end_loading (NautilusView *view,
+                                          gboolean all_files_seen)
+{
+	gboolean needs_reorganization;
+	gchar *stored_size_icon;
+	guint current_zoom;
+	guint current_icon_size;
+	gchar *current_icon_size_string;
+	NautilusFile *file;
+
+  	NAUTILUS_VIEW_CLASS (nautilus_desktop_canvas_view_parent_class)->end_loading (view, all_files_seen);
+
+        if (!all_files_seen)
+          return;
+
+  	file = nautilus_view_get_directory_as_file (view);
+  	g_return_if_fail (file != NULL);
+
+	stored_size_icon = nautilus_file_get_metadata (file, NAUTILUS_METADATA_KEY_DESKTOP_ICON_SIZE, NULL);
+	current_zoom = nautilus_canvas_container_get_zoom_level (get_canvas_container (view));
+	current_icon_size = nautilus_canvas_container_get_icon_size_for_zoom_level (current_zoom);
+	needs_reorganization = stored_size_icon == NULL || atoi (stored_size_icon) != current_icon_size;
+
+	if (needs_reorganization) {
+		current_icon_size_string = g_strdup_printf ("%d", current_icon_size);
+		nautilus_canvas_view_clean_up_by_name (NAUTILUS_CANVAS_VIEW (view));
+	  	nautilus_file_set_metadata (file, NAUTILUS_METADATA_KEY_DESKTOP_ICON_SIZE,
+                                            NULL, current_icon_size_string);
+
+	  	g_free (current_icon_size_string);
+	}
+
+	g_free (stored_size_icon);
+}
+
 static void
 nautilus_desktop_canvas_view_class_init (NautilusDesktopCanvasViewClass *class)
 {
@@ -268,6 +305,7 @@ nautilus_desktop_canvas_view_class_init (NautilusDesktopCanvasViewClass *class)
 
 	vclass->update_context_menus = real_update_context_menus;
 	vclass->get_view_id = real_get_id;
+	vclass->end_loading = nautilus_desktop_canvas_view_end_loading;
 
 	g_type_class_add_private (class, sizeof (NautilusDesktopCanvasViewDetails));
 }
@@ -329,17 +367,36 @@ get_default_zoom_level (void)
 	return CLAMP (default_zoom_level, NAUTILUS_CANVAS_ZOOM_LEVEL_SMALL, NAUTILUS_CANVAS_ZOOM_LEVEL_LARGE);
 }
 
+static void
+set_up_zoom_level (NautilusDesktopCanvasView *desktop_canvas_view)
+{
+	NautilusCanvasZoomLevel new_level;
+
+	new_level = get_default_zoom_level ();
+	nautilus_canvas_container_set_zoom_level (get_canvas_container (desktop_canvas_view),
+                                                  new_level);
+}
+
 static void
 default_zoom_level_changed (gpointer user_data)
 {
 	NautilusCanvasZoomLevel new_level;
 	NautilusDesktopCanvasView *desktop_canvas_view;
+	gint new_icon_size;
+	NautilusFile *file;
+	gchar *new_icon_size_string;
 
 	desktop_canvas_view = NAUTILUS_DESKTOP_CANVAS_VIEW (user_data);
+	file = nautilus_view_get_directory_as_file (NAUTILUS_VIEW (user_data));
 	new_level = get_default_zoom_level ();
+	new_icon_size = nautilus_canvas_container_get_icon_size_for_zoom_level (new_level);
+	new_icon_size_string = g_strdup_printf ("%d", new_icon_size);
 
-	nautilus_canvas_container_set_zoom_level (get_canvas_container (desktop_canvas_view),
-						new_level);
+	nautilus_file_set_metadata (file, NAUTILUS_METADATA_KEY_DESKTOP_ICON_SIZE,
+                                    NULL, new_icon_size_string);
+        set_up_zoom_level (desktop_canvas_view);
+
+	g_free (new_icon_size_string);
 }
 
 static gboolean
@@ -430,84 +487,6 @@ nautilus_desktop_canvas_view_update_canvas_container_fonts (NautilusDesktopCanva
 	g_free (font);
 }
 
-static void
-nautilus_desktop_canvas_view_init (NautilusDesktopCanvasView *desktop_canvas_view)
-{
-	NautilusCanvasContainer *canvas_container;
-	GtkAllocation allocation;
-	GtkAdjustment *hadj, *vadj;
-
-	desktop_canvas_view->details = G_TYPE_INSTANCE_GET_PRIVATE (desktop_canvas_view,
-								  NAUTILUS_TYPE_DESKTOP_CANVAS_VIEW,
-								  NautilusDesktopCanvasViewDetails);
-
-	if (desktop_directory == NULL) {
-		desktop_directory = nautilus_get_desktop_directory ();
-	}
-
-	canvas_container = get_canvas_container (desktop_canvas_view);
-	nautilus_canvas_view_container_set_sort_desktop (NAUTILUS_CANVAS_VIEW_CONTAINER (canvas_container), TRUE);
-
-	/* Do a reload on the desktop if we don't have FAM, a smarter
-	 * way to keep track of the items on the desktop.
-	 */
-	if (!nautilus_monitor_active ()) {
-		desktop_canvas_view->details->delayed_init_signal = g_signal_connect_object
-			(desktop_canvas_view, "begin-loading",
-			 G_CALLBACK (delayed_init), desktop_canvas_view, 0);
-	}
-	
-	nautilus_canvas_container_set_is_fixed_size (canvas_container, TRUE);
-	nautilus_canvas_container_set_is_desktop (canvas_container, TRUE);
-	nautilus_canvas_container_set_store_layout_timestamps (canvas_container, TRUE);
-
-	/* Set allocation to be at 0, 0 */
-	gtk_widget_get_allocation (GTK_WIDGET (canvas_container), &allocation);
-	allocation.x = 0;
-	allocation.y = 0;
-	gtk_widget_set_allocation (GTK_WIDGET (canvas_container), &allocation);
-	
-	gtk_widget_queue_resize (GTK_WIDGET (canvas_container));
-
-	hadj = gtk_scrollable_get_hadjustment (GTK_SCROLLABLE (canvas_container));
-	vadj = gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (canvas_container));
-
-	gtk_adjustment_set_value (hadj, 0);
-	gtk_adjustment_set_value (vadj, 0);
-
-	nautilus_view_ignore_hidden_file_preferences
-		(NAUTILUS_VIEW (desktop_canvas_view));
-
-	nautilus_view_set_show_foreign (NAUTILUS_VIEW (desktop_canvas_view),
-					FALSE);
-
-	g_signal_connect_object (canvas_container, "realize",
-				 G_CALLBACK (desktop_canvas_container_realize), desktop_canvas_view, 0);
-
-	g_signal_connect_object (desktop_canvas_view, "realize",
-				 G_CALLBACK (realized_callback), desktop_canvas_view, 0);
-	g_signal_connect_object (desktop_canvas_view, "unrealize",
-				 G_CALLBACK (unrealized_callback), desktop_canvas_view, 0);
-
-	g_signal_connect_swapped (nautilus_icon_view_preferences,
-				  "changed::" NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL,
-				  G_CALLBACK (default_zoom_level_changed),
-				  desktop_canvas_view);
-
-	g_signal_connect_swapped (nautilus_desktop_preferences,
-				  "changed::" NAUTILUS_PREFERENCES_DESKTOP_FONT,
-				  G_CALLBACK (font_changed_callback),
-				  desktop_canvas_view);
-
-	default_zoom_level_changed (desktop_canvas_view);
-	nautilus_desktop_canvas_view_update_canvas_container_fonts (desktop_canvas_view);
-
-	g_signal_connect_swapped (gnome_lockdown_preferences,
-				  "changed::" NAUTILUS_PREFERENCES_LOCKDOWN_COMMAND_LINE,
-				  G_CALLBACK (nautilus_view_update_context_menus),
-				  desktop_canvas_view);
-}
-
 static const gchar *
 get_control_center_command (const gchar ** params_out)
 {
@@ -653,11 +632,6 @@ real_update_context_menus (NautilusView *view)
 	view_action_group = nautilus_view_get_action_group (view);
 	desktop_view = NAUTILUS_DESKTOP_CANVAS_VIEW (view);
 
-	g_action_map_add_action_entries (G_ACTION_MAP (view_action_group),
-					 desktop_view_entries,
-					 G_N_ELEMENTS (desktop_view_entries),
-					 view);
-
 	action = g_action_map_lookup_action (G_ACTION_MAP (view_action_group), "empty-trash");
 	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), trash_link_is_selection (view));
 
@@ -684,6 +658,92 @@ real_update_context_menus (NautilusView *view)
 							       nautilus_canvas_container_is_stretched (canvas_container));
 }
 
+static void
+nautilus_desktop_canvas_view_init (NautilusDesktopCanvasView *desktop_canvas_view)
+{
+	NautilusCanvasContainer *canvas_container;
+	GtkAllocation allocation;
+	GActionGroup *view_action_group;
+	GtkAdjustment *hadj, *vadj;
+
+	desktop_canvas_view->details = G_TYPE_INSTANCE_GET_PRIVATE (desktop_canvas_view,
+								  NAUTILUS_TYPE_DESKTOP_CANVAS_VIEW,
+								  NautilusDesktopCanvasViewDetails);
+
+	if (desktop_directory == NULL) {
+		desktop_directory = nautilus_get_desktop_directory ();
+	}
+
+	canvas_container = get_canvas_container (desktop_canvas_view);
+	nautilus_canvas_view_container_set_sort_desktop (NAUTILUS_CANVAS_VIEW_CONTAINER (canvas_container), TRUE);
+
+	/* Do a reload on the desktop if we don't have FAM, a smarter
+	 * way to keep track of the items on the desktop.
+	 */
+	if (!nautilus_monitor_active ()) {
+		desktop_canvas_view->details->delayed_init_signal = g_signal_connect_object
+			(desktop_canvas_view, "begin-loading",
+			 G_CALLBACK (delayed_init), desktop_canvas_view, 0);
+	}
+
+	nautilus_canvas_container_set_is_fixed_size (canvas_container, TRUE);
+	nautilus_canvas_container_set_is_desktop (canvas_container, TRUE);
+	nautilus_canvas_container_set_store_layout_timestamps (canvas_container, TRUE);
+
+	/* Set allocation to be at 0, 0 */
+	gtk_widget_get_allocation (GTK_WIDGET (canvas_container), &allocation);
+	allocation.x = 0;
+	allocation.y = 0;
+	gtk_widget_set_allocation (GTK_WIDGET (canvas_container), &allocation);
+
+	gtk_widget_queue_resize (GTK_WIDGET (canvas_container));
+
+	hadj = gtk_scrollable_get_hadjustment (GTK_SCROLLABLE (canvas_container));
+	vadj = gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (canvas_container));
+
+	gtk_adjustment_set_value (hadj, 0);
+	gtk_adjustment_set_value (vadj, 0);
+
+	nautilus_view_ignore_hidden_file_preferences
+		(NAUTILUS_VIEW (desktop_canvas_view));
+
+	nautilus_view_set_show_foreign (NAUTILUS_VIEW (desktop_canvas_view),
+					FALSE);
+
+	g_signal_connect_object (canvas_container, "realize",
+				 G_CALLBACK (desktop_canvas_container_realize), desktop_canvas_view, 0);
+
+	g_signal_connect_object (desktop_canvas_view, "realize",
+				 G_CALLBACK (realized_callback), desktop_canvas_view, 0);
+	g_signal_connect_object (desktop_canvas_view, "unrealize",
+				 G_CALLBACK (unrealized_callback), desktop_canvas_view, 0);
+
+	g_signal_connect_swapped (nautilus_icon_view_preferences,
+				  "changed::" NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL,
+				  G_CALLBACK (default_zoom_level_changed),
+				  desktop_canvas_view);
+
+	g_signal_connect_swapped (nautilus_desktop_preferences,
+				  "changed::" NAUTILUS_PREFERENCES_DESKTOP_FONT,
+				  G_CALLBACK (font_changed_callback),
+				  desktop_canvas_view);
+
+	set_up_zoom_level (desktop_canvas_view);
+	nautilus_desktop_canvas_view_update_canvas_container_fonts (desktop_canvas_view);
+
+	g_signal_connect_swapped (gnome_lockdown_preferences,
+				  "changed::" NAUTILUS_PREFERENCES_LOCKDOWN_COMMAND_LINE,
+				  G_CALLBACK (nautilus_view_update_context_menus),
+				  desktop_canvas_view);
+
+	view_action_group = nautilus_view_get_action_group (NAUTILUS_VIEW (desktop_canvas_view));
+
+	g_action_map_add_action_entries (G_ACTION_MAP (view_action_group),
+					 desktop_view_entries,
+					 G_N_ELEMENTS (desktop_view_entries),
+					 NAUTILUS_VIEW (desktop_canvas_view));
+}
+
 NautilusView *
 nautilus_desktop_canvas_view_new (NautilusWindowSlot *slot)
 {
